//====================================================================================================
// File Name:   ov7670_drv.c
// Description: OV7670 CMOS Sensor driver
// Writer:      Jason Huang
// Last Modify: 2009/01/22
// Platform:    GPF32001A, unSP IDE V2.6.2, windows XP
// Version:     V1.1
//====================================================================================================
//====================================================================================================

//====================================================================================================
#include    "..\include\GPF32001A.h"
#include    "Sensor_include\sccb_lib.h"

//====================================================================================================
// Contant Defintion Area
//====================================================================================================
#define SCCB_SCL                        0x00    // IOA.0		
#define SCCB_SDA                        0x01    // IOA.1
#define SLAVE_ID                        0x42

#define C_TFT_CMOS_FrameBuffer_AddrL    0x0000
#define C_TFT_CMOS_FrameBuffer_AddrH    0x0003

//====================================================================================================
//  Description:     OV7670 Initialization
//  Syntax:          void OV7670_Init (
//                      int nWidthH,            // Active H Width
//                      int nWidthV,            // Active V Width
//                   );
//  Return:          None
//====================================================================================================
void OV7670_Init (
    int nWidthH,        // Active H Width
    int nWidthV         // Active V Width
) {
    unsigned int uCtrlReg1, uCtrlReg2;
    int nReso;

	
    //------------------------------------------------------------------------------------------------
    //  CMOS Sensor Interface (GF32001A) -
    //      CSI_CLKI    :   IOC7    (0x027)
    //      CSI_CLKO    :   IOC8    (0x028)
    //      CSI_HSYNC   :   IOC9    (0x029)
    //      CSI_VSYNC   :   IOC10   (0x02A)
    //      CSI_D0      :   IOA8    (0x08)
    //      CSI_D1      :   IOA9    (0x09)
    //      CSI_D2      :   IOA10   (0x0A)
    //      CSI_D3      :   IOA11   (0x0B)
    //      CSI_D4      :   IOA12   (0x0C)
    //      CSI_D5      :   IOA13   (0x0D)
    //      CSI_D6      :   IOA14   (0x0E)
    //      CSI_D7      :   IOA15   (0x0F)
    //------------------------------------------------------------------------------------------------

    // Set Sensor Driving Capability to 8 mA
    *P_IOC_Drv &= ~(0x0003 << C_LSL_IOC10_5_DRV);
    *P_IOC_Drv |= C_8mA << C_LSL_IOC10_5_DRV;
    // Set Sensor Input Schmit Trigger
    *P_IO_SR_SMT &= ~C_IOC_SR_Low;
    *P_IO_SR_SMT |= C_IOC_SMT_On;
    //

    // Setup sensor frame buffer start address
    *P_Sensor_TG_FrameBuf_AL = C_TFT_CMOS_FrameBuffer_AddrL;   // maps to TFT LCD frame buffer
    *P_Sensor_TG_FrameBuf_AH = C_TFT_CMOS_FrameBuffer_AddrH; 
    //
    // Setup sensor frame size
    // 640x480 / 320x240
    *P_Sensor_TG_H_Width = nWidthH;	        // Horizontal frame width
    *P_Sensor_TG_V_Width = nWidthV;	        // Vertical frame width

    //
    //	CMOS Sensor Interface (CSI) H/W Initialization
    //

    //
    // Enable CSI clock to let sensor initialize at first
    //
    *P_Sensor_TG_Ctrl2 = C_CSI_CLKO_En;  

    uCtrlReg2 = C_CLK_27M | C_CSI_CLKO_En | C_CLKO_Invert;  // Default CSI Control Register 2
    uCtrlReg1 = C_Capture | C_SensorEn | C_RGB_Output;   	// Default CSI Control Register 1

    if (0)     // CCIR656?
    {
        uCtrlReg1 |= C_CMOS_CCIR656; 
        uCtrlReg2 |= C_LatchDelay_2_Clk;  
    }
    else
    {
        uCtrlReg1 |= C_VADD_Rising | C_HRST_Rising | C_HREF_Mode;
        uCtrlReg2 |= C_LatchDelay_1_Clk;  
    }

    if (0)     // YUVIN?
    {
        uCtrlReg1 |= C_YUV_Input;  
    } 
    else 
    {
        uCtrlReg1 &= ~C_YUYV_GBGR;
        uCtrlReg2 |= C_LatchDelay_2_Clk; 
    }

    //
    // Whether compression or not?
    //
    nReso = ((nWidthH == 320) && (nWidthV == 240)) ? 0 : 1;
    {
        *P_Sensor_TG_H_CompRatio = 0x0000;
        *P_Sensor_TG_V_CompRatio = 0x0000;
        if (nReso) 
        {
            *P_Sensor_TG_H_CompRatio = 0x0102;   // Scale to 640*(1/2) = 320
            *P_Sensor_TG_V_CompRatio = 0x0102;   // Scale to 480*(1/2) = 240
        }
    }
    *P_Sensor_TG_V_L0Start = 0x0000;           // Sensor field 0 vertical latch start register.
    *P_Sensor_TG_V_L1Start = 0x0000;           // Sensor field 1 vertical latch start register.
    *P_Seneor_TG_H_LStart = 0x0000;            // Sensor horizontal start register.

    *P_Sensor_TG_Ctrl2 = uCtrlReg2;

    //
    // CMOS Sensor Initialization Start...
    //
    SCCB_Init(SCCB_SCL, SCCB_SDA);

    SCCB_Delay(200);
    SCCB_Write(SLAVE_ID, 0x12, 0x80);
    SCCB_Delay(200);

    SCCB_Write(SLAVE_ID, 0x11, 0x00);	// Use 1/2 External Clock Directly
    SCCB_Write(SLAVE_ID, 0x3a, 0x04);	// ??

    if (1)								// CCIR656?
    {
        SCCB_Write(SLAVE_ID, 0x04, 0x40);
    }
	
    if (nReso == 0) {
        if (0) {
            SCCB_Write(SLAVE_ID, 0x12, 0x10);	// QVGA + YUV Out
            if(SCCB_Read(SLAVE_ID, 0x12) != 0x10) while(1);
        } 
        else 
        {
            SCCB_Write(SLAVE_ID, 0x12, 0x14);	// QVGA + RGB Out
            if(SCCB_Read(SLAVE_ID, 0x12) != 0x14) while(1);
        }
        SCCB_Write(SLAVE_ID, 0x17, 0x16);	// HSTATT
        SCCB_Write(SLAVE_ID, 0x18, 0x04);	// HSTOP
        SCCB_Write(SLAVE_ID, 0x32, 0x80);	// HSTART/HSTOP LSB
        SCCB_Write(SLAVE_ID, 0x19, 0x02);	// VSTART
        SCCB_Write(SLAVE_ID, 0x1A, 0x7A);	// VSTOP
        SCCB_Write(SLAVE_ID, 0x03, 0x0A);	// VSTART/VSTOP LSB
        SCCB_Write(SLAVE_ID, 0x0C, 0x04);	// Sleep Control
        //SCCB_Write(SLAVE_ID, 0x3E, 0x19);	// PCLK Control
        SCCB_Write(SLAVE_ID, 0x3E, 0x00);	// PCLK Control
        SCCB_Write(SLAVE_ID, 0x70, 0x3A);	// H Scaler Control
        SCCB_Write(SLAVE_ID, 0x71, 0x35);	// V Scaler Control
        SCCB_Write(SLAVE_ID, 0x72, 0x11);	// Scaler DCR
        SCCB_Write(SLAVE_ID, 0x73, 0xf1);	// ??
	
    } 
    else if (nReso == 1) 
    {
        if (0) 
        {
            SCCB_Write(SLAVE_ID, 0x12, 0x00);	// VGA + YUV Out
            if(SCCB_Read(SLAVE_ID, 0x12) != 0x00) while(1);
        } 
        else 
        {
            SCCB_Write(SLAVE_ID, 0x12, 0x04);	// VGA + RGB Out
            if(SCCB_Read(SLAVE_ID, 0x12) != 0x04) while(1);
        }
        SCCB_Write(SLAVE_ID, 0x17, 0x13);	// HSTATT
        SCCB_Write(SLAVE_ID, 0x18, 0x01);	// HSTOP
        SCCB_Write(SLAVE_ID, 0x32, 0xB6);	// HSTART/HSTOP LSB
        SCCB_Write(SLAVE_ID, 0x19, 0x02);	// VSTART
        SCCB_Write(SLAVE_ID, 0x1A, 0x7A);	// VSTOP
        SCCB_Write(SLAVE_ID, 0x03, 0x0A);	// VSTART/VSTOP LSB
        SCCB_Write(SLAVE_ID, 0x0C, 0x00);	// Sleep Control
        SCCB_Write(SLAVE_ID, 0x3E, 0x00);	// PCLK Control
        SCCB_Write(SLAVE_ID, 0x70, 0x3A);	// H Scaler Control
        SCCB_Write(SLAVE_ID, 0x71, 0x35);	// V Scaler Control
        SCCB_Write(SLAVE_ID, 0x72, 0x11);	// Scaler DCR
        SCCB_Write(SLAVE_ID, 0x73, 0xf0);	// ??

    }

    SCCB_Write(SLAVE_ID, 0x73, 0xf0);	// ??
    SCCB_Write(SLAVE_ID, 0xA2, 0x02);	// Scaler PCLK Delay

    // Gamma Control
    SCCB_Write(SLAVE_ID, 0x7A, 0x24);
    SCCB_Write(SLAVE_ID, 0x7B, 0x04);
    SCCB_Write(SLAVE_ID, 0x7C, 0x07);
    SCCB_Write(SLAVE_ID, 0x7D, 0x10);
    SCCB_Write(SLAVE_ID, 0x7E, 0x28);
    SCCB_Write(SLAVE_ID, 0x7F, 0x36);
    SCCB_Write(SLAVE_ID, 0x80, 0x44);
    SCCB_Write(SLAVE_ID, 0x81, 0x52);
    SCCB_Write(SLAVE_ID, 0x82, 0x60);
    SCCB_Write(SLAVE_ID, 0x83, 0x6c);
    SCCB_Write(SLAVE_ID, 0x84, 0x78);
    SCCB_Write(SLAVE_ID, 0x85, 0x8c);
    SCCB_Write(SLAVE_ID, 0x86, 0x9e);
    SCCB_Write(SLAVE_ID, 0x87, 0xbb);
    SCCB_Write(SLAVE_ID, 0x88, 0xd2);
    SCCB_Write(SLAVE_ID, 0x89, 0xe5);

    // AEC/AGC Control
    SCCB_Write(SLAVE_ID, 0x13, 0xE0);
    SCCB_Write(SLAVE_ID, 0x00, 0x00);
    SCCB_Write(SLAVE_ID, 0x10, 0x00);
    SCCB_Write(SLAVE_ID, 0x0D, 0x50);
    if (nReso == 0)
        SCCB_Write(SLAVE_ID,0x14,0x38);
    else
        SCCB_Write(SLAVE_ID,0x14,0x18);
    SCCB_Write(SLAVE_ID, 0xA5, 0x05);
    SCCB_Write(SLAVE_ID, 0xAB, 0x07);
    SCCB_Write(SLAVE_ID, 0x24, 0x7f);
    SCCB_Write(SLAVE_ID, 0x25, 0x68);
    SCCB_Write(SLAVE_ID, 0x26, 0xc4);
    SCCB_Write(SLAVE_ID, 0x9F, 0x78);
    SCCB_Write(SLAVE_ID, 0xA0, 0x68);
    SCCB_Write(SLAVE_ID, 0xA1, 0x03);
    SCCB_Write(SLAVE_ID, 0xA6, 0xD8);
    SCCB_Write(SLAVE_ID, 0xA7, 0xD8);
    SCCB_Write(SLAVE_ID, 0xA8, 0xF0);
    SCCB_Write(SLAVE_ID, 0xA9, 0x90);
    SCCB_Write(SLAVE_ID, 0xAA, 0x94);
    SCCB_Write(SLAVE_ID, 0x13, 0xE5);

    // ??
    SCCB_Write(SLAVE_ID, 0x0E, 0x61);
    SCCB_Write(SLAVE_ID, 0x0F, 0x4B);
    SCCB_Write(SLAVE_ID, 0x16, 0x02);
    SCCB_Write(SLAVE_ID, 0x1E, 0x37);
    SCCB_Write(SLAVE_ID, 0x21, 0x02);
    SCCB_Write(SLAVE_ID, 0x22, 0x91);
    SCCB_Write(SLAVE_ID, 0x29, 0x07);
    SCCB_Write(SLAVE_ID, 0x33, 0x0B);
    SCCB_Write(SLAVE_ID, 0x35, 0x0B);
    SCCB_Write(SLAVE_ID, 0x37, 0x1D);
    SCCB_Write(SLAVE_ID, 0x38, 0x71);
    SCCB_Write(SLAVE_ID, 0x39, 0x2A);
    SCCB_Write(SLAVE_ID, 0x3C, 0x78);
    SCCB_Write(SLAVE_ID, 0x4D, 0x40);
    SCCB_Write(SLAVE_ID, 0x4E, 0x20);
    SCCB_Write(SLAVE_ID, 0x69, 0xaa);
    //SCCB_Write(SLAVE_ID, 0x6B, 0x4A);
    SCCB_Write(SLAVE_ID, 0x6B, 0x0A);	// Bypass PLL
    if (nReso == 0)
        SCCB_Write(SLAVE_ID, 0x74, 0x19);
    else
        SCCB_Write(SLAVE_ID,0x74,0x10);
    SCCB_Write(SLAVE_ID, 0x8D, 0x4F);
    SCCB_Write(SLAVE_ID, 0x8E, 0x00);
    SCCB_Write(SLAVE_ID, 0x8F, 0x00);
    SCCB_Write(SLAVE_ID, 0x90, 0x00);
    SCCB_Write(SLAVE_ID, 0x91, 0x00);
    SCCB_Write(SLAVE_ID, 0x96, 0x00);
    SCCB_Write(SLAVE_ID, 0x9A, 0x80);
    SCCB_Write(SLAVE_ID, 0xB0, 0x84);
    SCCB_Write(SLAVE_ID, 0xB1, 0x0C);
    SCCB_Write(SLAVE_ID, 0xB2, 0x0E);
    SCCB_Write(SLAVE_ID, 0xB3, 0x82);
    SCCB_Write(SLAVE_ID, 0xB8, 0x0A);

    // AWB Control
    if (nReso == 0)
        SCCB_Write(SLAVE_ID, 0x43, 0x14);
    else
        SCCB_Write(SLAVE_ID, 0x43, 0x0A);
    SCCB_Write(SLAVE_ID, 0x44, 0xF0);
    SCCB_Write(SLAVE_ID, 0x45, 0x34);
    SCCB_Write(SLAVE_ID, 0x46, 0x58);
    SCCB_Write(SLAVE_ID, 0x47, 0x28);
    SCCB_Write(SLAVE_ID, 0x48, 0x3A);
    SCCB_Write(SLAVE_ID, 0x59, 0x88);
    SCCB_Write(SLAVE_ID, 0x5A, 0x88);
    SCCB_Write(SLAVE_ID, 0x5B, 0x44);
    SCCB_Write(SLAVE_ID, 0x5C, 0x67);
    SCCB_Write(SLAVE_ID, 0x5D, 0x49);
    SCCB_Write(SLAVE_ID, 0x5E, 0x0E);
    SCCB_Write(SLAVE_ID, 0x6C, 0x0A);
    SCCB_Write(SLAVE_ID, 0x6D, 0x55);
    SCCB_Write(SLAVE_ID, 0x6E, 0x11);
    SCCB_Write(SLAVE_ID, 0x6F, 0x9f);
    SCCB_Write(SLAVE_ID, 0x6A, 0x40);
    SCCB_Write(SLAVE_ID, 0x01, 0x40);
    SCCB_Write(SLAVE_ID, 0x02, 0x40);
    SCCB_Write(SLAVE_ID, 0x13, 0xE7);

    // Matrix Coeficient
    SCCB_Write(SLAVE_ID, 0x4F, 0x80);
    SCCB_Write(SLAVE_ID, 0x50, 0x80);
    SCCB_Write(SLAVE_ID, 0x51, 0x00);
    SCCB_Write(SLAVE_ID, 0x52, 0x22);
    SCCB_Write(SLAVE_ID, 0x53, 0x5E);
    SCCB_Write(SLAVE_ID, 0x54, 0x80);
    SCCB_Write(SLAVE_ID, 0x58, 0x9E);

    // Edge Enhancement
    //SCCB_Write(SLAVE_ID, 0x41, 0x08);
    SCCB_Write(SLAVE_ID, 0x3F, 0x00);
    SCCB_Write(SLAVE_ID, 0x75, 0x05);
    SCCB_Write(SLAVE_ID, 0x76, 0xE1);
    SCCB_Write(SLAVE_ID, 0x4C, 0x00);
    SCCB_Write(SLAVE_ID, 0x77, 0x01);
    SCCB_Write(SLAVE_ID, 0x3D, 0xC2);
    SCCB_Write(SLAVE_ID, 0x4B, 0x09);
    SCCB_Write(SLAVE_ID, 0xC9, 0x60);
    //SCCB_Write(SLAVE_ID, 0x41, 0x38);
    SCCB_Write(SLAVE_ID, 0x56, 0x40);

    // 	Frame Rate Control
    SCCB_Write(SLAVE_ID, 0x34, 0x11);
    SCCB_Write(SLAVE_ID, 0x3B, 0x12);
    SCCB_Write(SLAVE_ID, 0xA4, 0x88);
    SCCB_Write(SLAVE_ID, 0x96, 0x00);
    SCCB_Write(SLAVE_ID, 0x97, 0x30);
    SCCB_Write(SLAVE_ID, 0x98, 0x20);
    SCCB_Write(SLAVE_ID, 0x99, 0x30);
    SCCB_Write(SLAVE_ID, 0x9A, 0x84);
    SCCB_Write(SLAVE_ID, 0x9B, 0x29);
    SCCB_Write(SLAVE_ID, 0x9C, 0x03);
    SCCB_Write(SLAVE_ID, 0x9D, 0x4C);
    SCCB_Write(SLAVE_ID, 0x9E, 0x3F);
    SCCB_Write(SLAVE_ID, 0x78, 0x04);

    SCCB_Write(SLAVE_ID, 0x79, 0x01);//base settings for 5060Hz auto detection
    SCCB_Write(SLAVE_ID, 0xc8, 0xf0);
    SCCB_Write(SLAVE_ID, 0x79, 0x0f);
    SCCB_Write(SLAVE_ID, 0xc8, 0x00);
    SCCB_Write(SLAVE_ID, 0x79, 0x10);
    SCCB_Write(SLAVE_ID, 0xc8, 0x7e);
    SCCB_Write(SLAVE_ID, 0x79, 0x0a);
    SCCB_Write(SLAVE_ID, 0xc8, 0x80);
    SCCB_Write(SLAVE_ID, 0x79, 0x0b);
    SCCB_Write(SLAVE_ID, 0xc8, 0x01);
    SCCB_Write(SLAVE_ID, 0x79, 0x0c);
    SCCB_Write(SLAVE_ID, 0xc8, 0x0f);
    SCCB_Write(SLAVE_ID, 0x79, 0x0d);
    SCCB_Write(SLAVE_ID, 0xc8, 0x20);
    SCCB_Write(SLAVE_ID, 0x79, 0x09);
    SCCB_Write(SLAVE_ID, 0xc8, 0x80);	
    SCCB_Write(SLAVE_ID, 0x79, 0x02);
    SCCB_Write(SLAVE_ID, 0xc8, 0xc0);
    SCCB_Write(SLAVE_ID, 0x79, 0x03);
    SCCB_Write(SLAVE_ID, 0xc8, 0x40);
    SCCB_Write(SLAVE_ID, 0x79, 0x05);
    SCCB_Write(SLAVE_ID, 0xc8, 0x30);		
    SCCB_Write(SLAVE_ID, 0x79, 0x26);		
 
    //brightness
    SCCB_Write(SLAVE_ID, 0x55, 0x80);//default

    //color saturation
    //1X
    SCCB_Write(SLAVE_ID, 0x4f, 0x80);
    SCCB_Write(SLAVE_ID, 0x50, 0x80);
    SCCB_Write(SLAVE_ID, 0x51, 0x00);
    SCCB_Write(SLAVE_ID, 0x52, 0x22);
    SCCB_Write(SLAVE_ID, 0x53, 0x5e);
    SCCB_Write(SLAVE_ID, 0x54, 0x80);
    SCCB_Write(SLAVE_ID, 0x58, 0x1a);

    //contrast	
    //default
    SCCB_Write(SLAVE_ID, 0x3c, 0x08);
    SCCB_Write(SLAVE_ID, 0x56, 0x40);
    SCCB_Write(SLAVE_ID, 0x57, 0x80);	

    //sharpness
    //Off
    //SCCB_Write(SLAVE_ID, 0x41, 0x00);
    SCCB_Write(SLAVE_ID, 0x41, 0x3f);
    //average window
    //full window
    SCCB_Write(SLAVE_ID, 0x0D, 0x00);
    SCCB_Write(SLAVE_ID, 0x42, 0x00);		

    *P_Sensor_TG_Ctrl1 = uCtrlReg1;
}


