//====================================================================================================
// File Name:   sccb_drv.c
// Description: SCCB protocol driver
// Writer:      Jason Huang
// Last Modify: 2009/01/22
// Platform:    GPF32001A, unSP IDE V2.6.2, windows XP
// Version:     V1.1
//====================================================================================================
//====================================================================================================
//
// SCCB Control C Code
// using SDA and SDL to control sensor interface
//
//====================================================================================================
#include "..\include\GPF32001A.h"

typedef struct
{
    unsigned int nData;
    unsigned int nBuffer;
    unsigned int nDir;
    unsigned int nAttrib;
    unsigned int nDrv;
} GPIO, *PGPIO;

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Global Variables for SCCB Library - begin
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
PGPIO PtrSCL;
PGPIO PtrSDA;
int SCL_BITNO;
int SDA_BITNO;

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Global Variables for SCCB Library - end
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++


//===================================================================
//  Description:    Delay function
//  Syntax:         SCCB_Delay (
//                      unsigned int i
//                  );
//  Return:         None
//===================================================================
void SCCB_Delay(
    unsigned int i
) {
    unsigned int j;

    for (j = 0; j < (i << 8); j++)
        asm ("nop");
}

//===================================================================
//  Description:    Start of data transmission
//  Function:       SCCB_Start
//  Syntax:         void SCCB_Start (void)
//  Input Paramter:
//                  none
//  Return:         none
//  Note:           Please refer to SCCB spec
//===================================================================
void SCCB_Start(void)
{
    PtrSCL->nBuffer |= (1 << SCL_BITNO);        //SCL1
    SCCB_Delay(1);
    PtrSDA->nBuffer |= (1 << SDA_BITNO);        //SDA1
    SCCB_Delay(1);
    PtrSDA->nBuffer &= ~(1 << SDA_BITNO);       //SDA0
    SCCB_Delay(1);
}

//===================================================================
//  Description:    Stop of data transmission
//  Function:       SCCB_Stop
//  Syntax:         void SCCB_Stop (void)
//  Input Paramter:
//                  none
//  Return:         none
//  Note:           Please refer to SCCB spec
//===================================================================
void SCCB_Stop(void)
{
    SCCB_Delay(1);
    PtrSDA->nBuffer &= ~(1 << SDA_BITNO);       //SDA0
    SCCB_Delay(1);
    PtrSCL->nBuffer |= (1 << SCL_BITNO);        //SCL1
    SCCB_Delay(1);
    PtrSDA->nBuffer |= (1 << SDA_BITNO);        //SDA1
    SCCB_Delay(1);
}

//===================================================================
//  Description:    Phase write process
//  Function:       SCCB_Write_Phase
//  Syntax:         void SCCB_Write_Phase (unsigned int value)
//  Input Paramter:
//                  unsigned int value:	output data
//  Return:         none
//  Note:           Please refer to SCCB spec
//===================================================================
void SCCB_Write_Phase(unsigned int value)
{
    int i;

    for (i = 0; i < 8; i++)
    {
        PtrSCL->nBuffer &= ~(1 << SCL_BITNO);       //SCL0
        SCCB_Delay(1);
        if (value & 0x80)
            PtrSDA->nBuffer |= (1 << SDA_BITNO);    //SDA1
        else
            PtrSDA->nBuffer &= ~(1 << SDA_BITNO);   //SDA0
        SCCB_Delay(1);
        PtrSCL->nBuffer |= (1 << SCL_BITNO);        //SCL1
        SCCB_Delay(2);
        value <<= 1;
    }
    // The 9th bit transmission
    PtrSCL->nBuffer &= ~(1 << SCL_BITNO);           //SCL0
    PtrSDA->nDir &= ~(1 << SDA_BITNO);              //SDA is Hi-Z mode
    SCCB_Delay(2);
    PtrSCL->nBuffer |= (1 << SCL_BITNO);            //SCL1
    SCCB_Delay(2);
    PtrSCL->nBuffer &= ~(1 << SCL_BITNO);           //SCL0
    PtrSDA->nDir |= (1 << SDA_BITNO);               //SDA is output mode
}

//===================================================================
//  Description:    Phase read process
//  Function:       SCCB_Read_Phase
//  Syntax:         unsigned int SCCB_Read_Phase (void)
//  Input Paramter: none
//  Return:         unsigned int: input data
//  Note:           Please refer to SCCB spec
//===================================================================
unsigned int SCCB_Read_Phase(void)
{
    int i;
    unsigned int data;

    PtrSDA->nDir &= ~(1 << SDA_BITNO);              //SDA is Hi-Z mode
    data = 0x00;
    for (i = 0; i < 8; i++)
    {
        PtrSCL->nBuffer &= ~(1 << SCL_BITNO);       //SCL0
        SCCB_Delay(2);
        PtrSCL->nBuffer |= (1 << SCL_BITNO);        //SCL1
        data = (data <<= 1) | ((PtrSDA->nData & (1 << SDA_BITNO)) >> SDA_BITNO);
        SCCB_Delay(2);
    }
    // The 9th bit transmission
    PtrSCL->nBuffer &= ~(1 << SCL_BITNO);           //SCL0
    PtrSDA->nDir |= (1 << SDA_BITNO);               //SDA is output mode
    PtrSDA->nBuffer |= (1 << SDA_BITNO);            //SDA1
    SCCB_Delay(2);
    PtrSCL->nBuffer |= (1 << SCL_BITNO);            //SCL1
    SCCB_Delay(2);
    PtrSCL->nBuffer &= ~(1 << SCL_BITNO);           //SCL0
    return data;		
}

//====================================================================================================
//  Description:    SCCB Initialization
//  Syntax:         void SCCB_Init (
//                      int nSCL,				// Clock Port No
//                                                  Ex. PORTA.9 = 0x09	/ PORTB.3 = 0x13
//                      int nSDA				// Data Port No
//                                                  Ex. PORTA.10 = 0x0A	/ PORTB.4 = 0x14
//                  );
//  Return:         None
//====================================================================================================
void SCCB_Init(
    int nSCL,               // Clock Port No
    int nSDA                // Data Port No
) {
    PtrSCL = (PGPIO)(P_IOA_Data + ((nSCL & 0x70) >> 1));
    SCL_BITNO = nSCL & 0x0F;
    PtrSDA = (PGPIO)(P_IOA_Data + ((nSDA & 0x70) >> 1));
    SDA_BITNO = nSDA & 0x0F;
}

//====================================================================================================
//  Description:    SCCB write register process
//  Syntax:         void SCCB_Write (
//                      unsigned int id,                // Slave ID
//                      unsigned int addr,              // Register Address
//                      unsigned int data               // Register Data
//                  );
//  Return:         None
//  Note:           Please refer to SCCB spec
//====================================================================================================
void SCCB_Write(
    unsigned int id,                // Slave ID
    unsigned int addr,              // Register Address
    unsigned int data               // Register Data
) {
    // Data re-verification
    id &= 0xFF;
    addr &= 0xFF;
    data &= 0xFF;

    // Serial bus output mode initialization
    PtrSCL->nAttrib = PtrSCL->nAttrib | (1 << SCL_BITNO);
    PtrSDA->nAttrib = PtrSDA->nAttrib | (1 << SDA_BITNO);
    PtrSCL->nDir = PtrSCL->nDir | (1 << SCL_BITNO);
    PtrSDA->nDir = PtrSDA->nDir | (1 << SDA_BITNO);

    // 3-Phase write transmission cycle is starting now ...
    SCCB_Start();                              // Transmission start
    SCCB_Write_Phase(id);                      // Phase 1
    SCCB_Write_Phase(addr);                    // Phase 2
    SCCB_Write_Phase(data);                    // Phase 3
    SCCB_Stop();                               // Transmission stop
}

//====================================================================================================
//  Description:    SCCB read register process
//  Syntax:         unsigned int SCCB_Read (
//                      unsigned int id,                // Slave ID
//                      unsigned int addr               // Register Address
//                  );
//  Input Paramter:
//                  unsigned int id     : slave device id
//                  unsigned int addr   : register address
//  Return:
//                  unsigned int        : register data
//  Note:           Please refer to SCCB spec
//====================================================================================================
unsigned int SCCB_Read(
    unsigned int id,                // Slave ID
    unsigned int addr               // Register Address
) {
    unsigned int data;
	
    // Data re-verification
    id &= 0xFF;
    addr &= 0xFF;

    // Serial bus output mode initialization
    PtrSCL->nAttrib = PtrSCL->nAttrib | (1 << SCL_BITNO);
    PtrSDA->nAttrib = PtrSDA->nAttrib | (1 << SDA_BITNO);
    PtrSCL->nDir = PtrSCL->nDir | (1 << SCL_BITNO);
    PtrSDA->nDir = PtrSDA->nDir | (1 << SDA_BITNO);

    // 2-Phase write transmission cycle is starting now ...
    SCCB_Start();                              // Transmission start
    SCCB_Write_Phase(id);                      // Phase 1
    SCCB_Write_Phase(addr);                    // Phase 2
    SCCB_Stop ();                              // Transmission stop

    // 2-Phase read transmission cycle is starting now ...
    SCCB_Start();                              // Transmission start
    SCCB_Write_Phase(id | 0x01);               // Phase 1 (read)
    data = SCCB_Read_Phase();                  // Phase 2
    SCCB_Stop();                               // Transmission stop
    return data;
}