//==========================================================================
// Description: Example code of COMS Sensor Interface, capture image and send to TFT display in real time
// Writer:      Jason Huang
// Date:        2009/01/22
// Platform:    GPF32001A, unSP IDE V2.6.2, windows XP
// Version:     V1.1
//==========================================================================
//**************************************************************************
// Header File Included Area
//**************************************************************************
#include  "..\include\GPF32001A.h"

//**************************************************************************
// Contant Defintion Area
//**************************************************************************
#define C_SPI_CS_BIT                    0x0010     // Set IOB.4 as CS pin

#define C_TFT_CMOS_FrameBuffer_AddrL    0x0000
#define C_TFT_CMOS_FrameBuffer_AddrH    0x0003

//**************************************************************************
// Function Call Publication Area
//**************************************************************************
void System_ConfigMemory(void);
void System_InitCache(void);
void System_InitClock(void);
void TFT_SetCommand(int command);

void OV7670_Init (
    int nWidthH,        // Active H Width
    int nWidthV         // Active V Width
);

//**************************************************************************
// Global Variable Defintion Area
//**************************************************************************

//**************************************************************************
// Main Program
//**************************************************************************
int main()
{
    System_ConfigMemory();
    System_InitCache();
    System_InitClock();

    OV7670_Init(320, 240);

    *P_PPU_Enable = C_FB_RGB565_RGBG | C_FB_Format_RGB565 | C_FrameBaseMode | C_QVGA | C_BottomToTop; 

    TFT_SetCommand(0x0800);         // See TD025THEA7 TFT LCD documents for reference

    *P_TFT_RGB_Ctrl = C_Even_RGB | C_Odd_RGB;
    *P_TFT_V_Width = 262;           // TFT Vsync timing setup	
    *P_TFT_V_Start = 20;
    *P_TFT_V_End = 20 + 240;
    *P_TFT_H_Width = 1560;          // TFT Hsync timing setup
    *P_TFT_H_Start = 240;
    *P_TFT_H_End = 240 + 1280;
    *P_TFT_Ctrl = C_VSU_DCLK | C_HSYNC_Invert | C_VSYNC_Invert | C_UPS052 | C_TFT_CLK_SysClk_Div_5 | C_TFT_Enable; 

    *P_FBI_AddrL = C_TFT_CMOS_FrameBuffer_AddrL;  // FTF frame buffer address, maps to CMOS Sensor frame buffer
    *P_FBI_AddrH = C_TFT_CMOS_FrameBuffer_AddrH;

    while (1)
    {
    }

    return 0;
}


//**************************************************************************
// Function     : void System_ConfigMemory(void)
// Description  : Configure system memory
// Parameter    : None
// Return       : None
// Note         : None
//**************************************************************************

void System_ConfigMemory(void)
{
    *P_MAPSEL = C_CS0_BootMap_4K; 
    
    *P_MCS0_Ctrl = 0x0300 | C_Access_SRAM | C_Wait_8_SysClk;      // CS0 SRAM 4*64K word
    *P_MCS1_Ctrl = 0x0000 | C_Access_SRAM | C_Wait_8_SysClk;      // CS1 SRAM 1*64K word
    *P_MCS2_Ctrl = 0x0000 | C_Access_SRAM | C_Wait_8_SysClk;      // CS2 SRAM 1*64K word
    *P_MCS3_Ctrl = 0x0000 | C_Access_SRAM | C_Wait_8_SysClk;      // CS3 SRAM 1*64K word
    *P_MCS4_Ctrl = 0x0000 | C_Access_SRAM | C_Wait_8_SysClk;      // CS4 SRAM 1*64K word
    
    *P_IOD_Drv = (C_16mA << C_LSL_IOD7_DRV)  |  \
                 (C_16mA << C_LSL_IOD8_DRV)  |  \
                 (C_16mA << C_LSL_IOD9_DRV)  |  \
                 (C_16mA << C_LSL_IOD10_DRV) |  \
                 (C_16mA << C_LSL_IOD15_11_DRV);
    
    *P_MEM_Drv = (C_16mA << C_LSL_MEM_XD15_0_DRV) |  \
                 (C_16mA << C_LSL_MEM_XA7_0_DRV)  |  \
                 (C_16mA << C_LSL_MEM_XA14_8_DRV) |  \
                 (C_16mA << C_LSL_MEM_BKWEB_DRV)  |  \
                 (C_16mA << C_LSL_MEM_BKOEB_DRV);
    
    *P_MCS_Drv = (C_16mA << C_LSL_CS0_DRV) |  \
                 (C_16mA << C_LSL_CS1_DRV) |  \
                 (C_16mA << C_LSL_CS2_DRV) |  \
                 (C_16mA << C_LSL_CS3_DRV) |  \
                 (C_16mA << C_LSL_CS4_DRV);
}

//**************************************************************************
// Function     : void System_InitCache(void)
// Description  : Initialize instruction cache
// Parameter    : None
// Return       : None
// Note         : None
//**************************************************************************
void System_InitCache(void)
{
    unsigned int temp;

    *P_Cache_Ctrl = C_Cache_Clear; 

    temp = *P_Cache_Ctrl;		
    while (temp & C_Cache_Clear)  // Wait until cache clear is completed
    {
        temp = *P_Cache_Ctrl;	
    }

    *P_Cache_Ctrl = C_Cache_Enable | C_Cache_4K_Byte | C_Cache_BurstAccess;

    temp = *P_Cache_Ctrl;
    while (!(temp & C_Cache_Enable))  // Ensure the cache is really enabled
    {
        temp = *P_Cache_Ctrl;
    }
}

//**************************************************************************
// Function     : void System_InitClock(void)
// Description  : Initialize system clock
// Parameter    : None
// Return       : None
// Note         : None
//**************************************************************************
void System_InitClock(void)
{
    unsigned int temp;

    temp = *P_Power_State;
    while (!(temp & C_State_Bits))      // Wait clock stable
    {
        temp = *P_Power_State;
    }

    *P_Clock_Ctrl &= ~C_FastPLL;        // Disable fast PLL

    temp = *P_Power_State;
    while ((temp & C_State_Bits) != C_State_SlowClock)   // Wait clock stable
    {
        temp = *P_Power_State;
    }

    *P_PLLChange = C_PLL_96M;           // Set PLL clock as 96MHz
    *P_Clock_Ctrl |= C_FastPLL | C_IOB6_GPIO | C_27MHz_Enable;    

    temp = *P_Power_State;			
    while ((temp & C_State_Bits) != C_State_FastClock)   // Wait clock stable
    {
        temp = *P_Power_State;
    }  
}

//**************************************************************************
// Function     : void TFT_SetCommand(void)
// Description  : Send command to TFT module by SPI interface
// Parameter    : None
// Return       : None
// Note         : None
//**************************************************************************
void TFT_SetCommand(int command)
{
    unsigned int temp;

    // CM1: TFT_SCL, SPI_CLK
    // CM2: TFT_SDA, SPI_TXD
    // CM3: TFT_CS,  SPI_CS0

    *P_IOB_Dir = C_SPI_CS_BIT;      // Set CS pin output high in default
    *P_IOB_Attrib = C_SPI_CS_BIT;
    *P_IOB_Data = C_SPI_CS_BIT;


    *P_SPI_Ctrl = C_SPI_SoftReset;    
    *P_SPI_Ctrl = C_SPI_Enable | C_SPI_MasterMode | C_SPI_SysClk_Div_128;       
    *P_SPI_Misc = C_SPI_FIFO_SmartMode;    

    *P_IOB_Buffer &= ~C_SPI_CS_BIT; // CS is low

    *P_SPI_TX_Data = command >> 8;  // TX data
    *P_SPI_TX_Data = command;

    temp = *P_SPI_RX_Status;
    while (!(temp & 0x0002))        // Wait for data received
    {
        temp = *P_SPI_RX_Status;
    }

    *P_IOB_Buffer |= C_SPI_CS_BIT;  // CS is high

    temp = *P_SPI_RX_Data;          // Clear RX buffer
    temp = *P_SPI_RX_Data;
}