//==========================================================================
// File Name   : ISR.asm
// Description : Interrupt Service Routine
// Written by  : Jason Huang
// Last modified date:
//               2009/01/22
// Note: 
// 1. Interrupts must be declared in TEXT area.
// 2. For FIQ, IRQ1 ~ IRQ7, user needs to clear P_INT_Clear before 
//    exiting interrupt routine
// 3. 
// 	_FIQ:   FIQ                                                 // Fast interrupt entry
//  _IRQ0:  Audio Channel A FIFO Empty                          // interrupt entry
//          Audio Channel B FIFO Empty
//  _IRQ1:  ADC Auto Sampling FIFO Full                         // interrupt entry
//          AD Conversion Ready 
//  _IRQ2:  External A                                          // interrupt entry
//          External B
//  _IRQ3:  UART/IrDA                                           // interrupt entry
//          Serial Peripheral Interface(SPI)
//          DMA Transfer 
//          USB
//  _IRQ4:  TimerA Up-Counter Overflow/Capture/Comparison event // interrupt entry
//          TimerB Up-Counter Overflow/Capture/Comparison event
//          TimerC Up-Counter Overflow/Capture/Comparison event
//          TimerD Up-Counter Overflow
//          SPU
//  _IRQ5:  Key Change                                          // interrupt entry
//          LCD Frame Pulse Rising Edge(FP)
//          TFT Under Flow Error
//          TFT Frame End
//          SDC1 Controller
//          NAND Flash FIFO over/under flow
//          PPU
//  _IRQ6:  Key-Scan                                            // interrupt entry     
//          TimeBaseC
//          Scheduler
//          SDC2 Controller
//  _IRQ7:  TimeBaseA                                           // interrupt entry
//          TimeBaseB
//          Alarm
//          Hour/Minute/Second/Half-Second
//          Unexpected memory area accessing
//  _Break: Software interrupt                                  // interrupt entry
//==========================================================================
//**************************************************************************
// Header File Included Area
//**************************************************************************
.include ..\include\GPF32001A.inc

//**************************************************************************
// Contant Defintion Area
//**************************************************************************

//**************************************************************************
// Variable Publication Area
//**************************************************************************
.external _Key, _PreKey; 

//**************************************************************************
// Function Call Publication Area
//**************************************************************************
.public _BREAK
.public _FIQ
.public _IRQ0
.public _IRQ1
.public _IRQ2
.public _IRQ3
.public _IRQ4
.public _IRQ5
.public _IRQ6
.public _IRQ7

//**************************************************************************
// External Variable Declaration
//**************************************************************************

//**************************************************************************
// External Function Declaration
//**************************************************************************

//**************************************************************************
// External Table Declaration
//**************************************************************************

//**************************************************************************
// RAM Definition Area
//**************************************************************************

//**************************************************************************
// Table Definition Area
//**************************************************************************

//**************************************************************************
// CODE Definition Area
//**************************************************************************
.text
_FIQ:
    push R1, R5 to [SP];	
  	
    pop R1, R5 from [sp];	
    reti;
//**************************************************
//
//**************************************************
_BREAK:        
    push R1, R5 to [SP];

    pop R1, R5 from [SP];
    reti; 
 //**************************************************
//
//**************************************************       
_IRQ0:        
    push R1, R5 to [SP];

    pop R1, R5 from [SP];
    reti; 
//**************************************************
//
//**************************************************        
_IRQ1:        
    push R1, R5 to [SP];

    pop R1, R5 from [SP];
    reti;  
//**************************************************
//
//**************************************************        
_IRQ2:        
    push R1, R5 to [SP];

    pop R1, R5 from [SP];
    reti;         
//**************************************************
//
//**************************************************        
_IRQ3:        
    push R1, R5 to [SP];

    pop R1, R5 from [SP];
    reti;         
//**************************************************
//
//**************************************************        
_IRQ4:        
    push R1, R5 to [SP];

    R1 = [P_INT_Status2];
    test R1, C_INT_TimerC;       
    jz ?L_ExitIRQ4;
    
    R1 = [P_TimerC_Ctrl];   // Clear TimerC interrupt flag
    [P_TimerC_Ctrl] = R1;

    R1 = [P_KS_Ctrl2];
    R1 |= C_KeyPressed;     // Enable read A, B key status independly to check key release
    [P_KS_Ctrl2] = R1;

    BP = P_KS_Data0;
    R4 = _PreKey;
    R3 = 0;
?L_CheckKeyStatus:	
    R1 = [BP];
    R1 & = 0x00FF;          // Only check A or B key status
    cmp R1, [R4];
    je ?L_NextKeyStatus
    R2 = [R4];
    [R4] = R1;
    R1 ^= R2;               // Changed key
    R2 &= R1;               // Released key status

    // Calculate the release key number
    R1 = 0;
?L_CheckReleaseKey:
    test R2, 0x0001;
    jz ?L_NextReleaseBit;
    call F_GetReleaseKey;
?L_NextReleaseBit:
    R2 = R2 lsr 1;
    R1 += 1;
    cmp R1, 8;
    jb ?L_CheckReleaseKey;	

?L_NextKeyStatus:
    BP += 1;
    R4 += 1;
    R3 += 1;
    cmp R3, 8;
    jb ?L_CheckKeyStatus;	

    R1 = [P_KS_Ctrl2];
    R1 &= ~C_KeyPressed;    // Disable read A, B key status independly
    [P_KS_Ctrl2] = R1;

?L_ExitIRQ4:	
    pop R1, R5 from [SP];
    reti;  
    
//**************************************************
//
//**************************************************        
_IRQ5:        
    push R1, R5 to [SP];

    pop R1, R5 from [SP];
    reti;  
           
//**************************************************
//
//**************************************************        
_IRQ6:        
    push R1, R5 to [SP];

    R4 = 0;
    BP = P_KS_Data0;
?L_CheckNextLine:	
    R1 = [BP];          // Get A&B key status 
    jz ?L_NextLine;
    [BP] = R1;          // Clear A&B key status
    // Check velocity key and get velocity value after key number calculation
    R3 = 8;				
    MR = R4 * R3;
    R2 = 0;
?L_CheckPressKey:
    test R1, 0x0001;
    jz ?L_NextBit;
    call F_ReadVelocity;
?L_NextBit:
    R1 = R1 lsr 1;
    R2 += 1;
    cmp R2, 8;
    jb ?L_CheckPressKey;	
?L_NextLine:
    BP += 1;
    R4 += 1;
    cmp R4, 8
    jb ?L_CheckNextLine;
	
?L_NoNewKeyPressed:
    R1 = [P_KS_Ctrl1];  // Clear key scan interrupt flag
    [P_KS_Ctrl1] = R1;

?L_Exit_IRQ6:
    pop R1, R5 from [SP];
    reti;         
        
//**************************************************
//
//**************************************************        
_IRQ7:        
    push R1, R5 to [SP];

    pop R1, R5 from [SP];
    reti;


//**************************************************
//
//**************************************************
.code

//**************************************************
// Function     : F_ReadVelocity
// Description  : Get velocity value
// Parameter    : R2: Line number of key matrix
//                R3: Column number of key matrix
// Return       : None
// Note         : None
//**************************************************
F_ReadVelocity:     .proc
    push R2 to [SP];

    R2 += R3;               // Stroke key number
    [_Key] = R2;
    R2 |= C_ReadVelocity;   // Enable read velocity
    [P_KS_Addr] = R2;

?L_WaitVelocityData:
    R2 = [P_KS_Addr];
    test R2, C_ReadVelocity;
    jnz ?L_WaitVelocityData;

    R2 = [P_KS_Velocity];   // Get velocity data	

    pop R2 from [SP];
    retf;
    .endp
	
//**************************************************
// Function     : F_GetReleaseKey
// Description  : Calculate release key code
// Parameter    : R1: Line number of key matrix
//                R3: Column number of key matrix
// Return       : Released Key Code = Pressed Key Code + 128
// Note         : None
//**************************************************	
F_GetReleaseKey:    .proc
    push R3, R4 to [SP];

    R4 = 8;
    MR = R3 * R4;
    R3 += R1;
    R3 += 0x80;             // Release key number
    [_Key] = R3;

    pop R3, R4 from [SP];
    retf;
    .endp
	
	
